
/* 
 * file_sel.c --a single pushbutton, when selected, creates a
 * file selection dialog that displays a list of all the writable
 * files in the directory described by the XmNmask of the dialog.
 * This program demonstrates how to use the XmNfileSearchProc for
 * file selection dialog widgets.
 *
 */

#define index strchr

#include "unix.h"
#include "x.h"
#include "xm.h"
#include "off.h"
#include "externs.h"

#define OPEN_HELP    401
#define NEW_HELP    402

XmStringCharSet charset = XmSTRING_DEFAULT_CHARSET;

/* routined to determine if a file is accessible, a directory,
 * or writable.  Return -1 on all errors or if the file is not
 * writable.  Return 0 if it's a directory or 1 if it's a plain
 * writable file.
 */
int
is_writable( file, ext )
char *file, *ext;
{
    char   *ptr;
    struct  stat s_buf;
    int     lf = strlen(file);

    /* if file can't be accessed ( via stat(  ) ) return. */
    if ( stat( file, &s_buf ) == -1 )
        return -1;
    else if ( ( s_buf.st_mode & S_IFMT ) == S_IFDIR )
        return 0; /* a directory */
    else if ( ( s_buf.st_mode & S_IEXEC ) == S_IEXEC )
        return -1; /* a executable */
    else if ( !( s_buf.st_mode & S_IFREG ) || access( file, W_OK ) == -1 )
        /* not a normal file or it is not writable */
        return -1;
    else if( ( file[lf-1] != ext[3] ||
               file[lf-2] != ext[2] ||
               file[lf-3] != ext[1] ||
               file[lf-4] != ext[0] ) )
        return -1;
#if( 0 ) 
    else if( (ptr = index( file, '.' )) != NULL && 
        ptr++ &&
        strcmp( ptr, "aoff" ) )
        return -1;
    else if( (ptr = index( file, 'a' )) != NULL && 
        strcmp( ptr, "aoff" ) )
        return -1;
#endif		/*  */
    /* legitimate file */
    return 1;
}


Boolean XmStringGetLtoR( string, charset, text )
XmString string;
XmStringCharSet charset;
char **text;
{
    XmStringContext context;
    char * t;
    XmStringCharSet c, curcharset = NULL;
    XmStringDirection d;
    Boolean s, is_local = FALSE, done = FALSE, is_default = FALSE;
    extern char *_XmStringGetCurrentCharset();

    if (!string) return(FALSE);
    if (!charset) return (FALSE);

    if ((charset == XmFONTLIST_DEFAULT_TAG) ||
    (strcmp(charset, XmFONTLIST_DEFAULT_TAG) == 0))
      is_local = TRUE;

    *text = NULL;               /* pre-condition result */

    if (!is_local)
      {
      if ((strcmp(charset, XmSTRING_DEFAULT_CHARSET) == 0))
        {
          curcharset = _XmStringGetCurrentCharset();
          is_default = TRUE;
        }
      else curcharset = charset;
      }

    XmStringInitContext (&context, string);

    while ( ! done)
    {
    if (XmStringGetNextSegment (context, &t, &c, &d, &s))
    {
            if (c && (d == XmSTRING_DIRECTION_L_TO_R) &&
                (((is_local || is_default) &&
                  ((c == XmFONTLIST_DEFAULT_TAG) ||
           (strcmp(c, XmFONTLIST_DEFAULT_TAG) == 0) ||
           (strcmp(c, _XmStringGetCurrentCharset()) == 0))) ||
         (curcharset && (strcmp (c, curcharset) == 0))))
        {
        *text = t;          /* OK, pass text to caller */
        done = TRUE;
        }
        else
        XtFree (t);         /* not this text */

        if (c)
                XtFree (c);                     /* always dump charset */
    }
    else
        done = TRUE;
    }

    XmStringFreeContext (context);
    return (*text != NULL);
}


/* do_search(  ) -- scan a directory and report only those files that
 * are writable.  Here, we let the shell expand the ( possible )
 * wildcards and return a directory listing by using popen(  ).
 * A *real* application should -not- do this; it should use the
 * system's directory routines: opendir(  ), readdir(  ) and closedir(  ).
 */
void
do_search( fs, cbs )
Widget fs; /* file selection box widget */
XmFileSelectionBoxCallbackStruct *cbs;
{
    char          *mask, buf[BUFSIZ], *p;
    XmString       names[256]; /* maximum of 256 files in dir */
    int            i = 0;
    FILE          *pp, *popen(  );
    extern char   *pattern;

    if ( !XmStringGetLtoR( cbs->mask, charset, &mask ) )
        return; /* can't do anything */

    sprintf( buf, "/bin/ls %s", mask );
    XtFree( mask );
    /* let the shell read the directory and expand the filenames */
    if ( !( pp = popen( buf, "r" ) ) )
        return;
    /* read output from popen(  ) -- this will be the list of files */
    while ( fgets( buf, sizeof buf, pp ) ) {
        if ( p = index( buf, '\n' ) )
            *p = 0;
        /* only list files that are writable and not directories */
        if ( is_writable( buf, pattern ) == 1 &&
            ( names[i] = XmStringCreateSimple( buf ) ) )
            i++;
    }
    pclose( pp );
    if ( i ) {
        XtVaSetValues( fs,
            XmNfileListItems,      names,
            XmNfileListItemCount,  i,
            XmNdirSpec,            names[0],
            XmNlistUpdated,        True,
            NULL );
        while ( i > 0 )
            XmStringFree( names[--i] );
    } else
        XtVaSetValues( fs,
            XmNfileListItems,      NULL,
            XmNfileListItemCount,  0,
            XmNlistUpdated,        True,
            NULL );
}
